@@ -491,9 +491,9 @@ int main (int argc, char **argv)
 
 	if (totem_embedded_open (emb) != FALSE)
 		totem_embedded_play (emb);
-g_print ("enter mainloop\n");
+
 	gtk_main ();
-g_print ("exit\n");
+
 	return 0;
 }
 
