@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
- * Copyright (C) 2001-2004 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2001-2005 Bastien Nocera <hadess@hadess.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <gnome.h>
 #include "totem-gromit.h"
 #else
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
@@ -52,14 +53,12 @@
 #include "totem-time-label.h"
 #include "totem-session.h"
 #include "totem-screenshot.h"
-#include "totem-sublang.h"
+#include "totem-menu.h"
 #include "totem-options.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
 #include "video-utils.h"
 
-#include "egg-recent-view.h"
-
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
@@ -181,6 +180,7 @@ totem_action_exit (Totem *totem)
 	if (totem->win)
 		gtk_widget_hide (totem->win);
 
+	totem_sublang_exit (totem);
 	totem_named_icons_dispose (totem);
 
 	if (totem->bvw)
@@ -277,6 +277,7 @@ totem_action_eject (Totem *totem)
 	GError *err = NULL;
 	char *cmd, *prefix;
 	const char *needle;
+	char *device;
 
 	needle = strchr (totem->mrl, ':');
 	g_assert (needle != NULL);
@@ -285,8 +286,11 @@ totem_action_eject (Totem *totem)
 	totem_playlist_clear_with_prefix (totem->playlist, prefix);
 	g_free (prefix);
 
-	cmd = g_strdup_printf ("eject %s", gconf_client_get_string
-			(totem->gc, GCONF_PREFIX"/mediadev", NULL));
+	g_object_get (G_OBJECT (totem->bvw),
+			"mediadev", &device, NULL);
+	cmd = g_strdup_printf ("eject %s", device);
+	g_free (device);
+
 	if (g_spawn_command_line_sync (cmd, NULL, NULL, NULL, &err) == FALSE)
 	{
 		totem_action_error (_("Totem could not eject the optical media."), err->message, totem);
@@ -304,6 +308,9 @@ totem_action_play (Totem *totem)
 	if (totem->mrl == NULL)
 		return;
 
+	if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+		return;
+
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
 	if (totem_is_fullscreen (totem) != FALSE)
@@ -352,8 +359,8 @@ totem_action_seek (Totem *totem, double pos)
 	}
 }
 
-static void
-totem_action_set_mrl_and_play (Totem *totem, char *mrl)
+void
+totem_action_set_mrl_and_play (Totem *totem, const char *mrl)
 {
 	if (totem_action_set_mrl (totem, mrl) != FALSE)
 		totem_action_play (totem);
@@ -362,7 +369,7 @@ totem_action_set_mrl_and_play (Totem *totem, char *mrl)
 static gboolean
 totem_action_load_media (Totem *totem, MediaType type)
 {
-	const char **mrls;
+	char **mrls;
 	char *msg;
 
 	if (bacon_video_widget_can_play (totem->bvw, type) == FALSE)
@@ -382,7 +389,9 @@ totem_action_load_media (Totem *totem, MediaType type)
 		return FALSE;
 	}
 
-	totem_action_open_files (totem, (char **)mrls);
+	totem_action_open_files (totem, mrls);
+	g_strfreev (mrls);
+
 	return TRUE;
 }
 
@@ -439,6 +448,17 @@ totem_action_play_pause (Totem *totem)
 	}
 }
 
+void
+totem_action_pause (Totem *totem)
+{
+	if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+	{
+		bacon_video_widget_pause (totem->bvw);
+		play_pause_set_label (totem, STATE_PAUSED);
+		totem_scrsaver_enable (totem->scr);
+	}
+}
+
 void
 totem_action_fullscreen_toggle (Totem *totem)
 {
@@ -497,14 +517,14 @@ totem_get_nice_name_for_stream (Totem *totem)
 	GValue value = { 0, };
 
 	bacon_video_widget_get_metadata (totem->bvw, BVW_INFO_TITLE, &value);
-	title = g_strdup (g_value_get_string (&value));
+	title = g_value_dup_string (&value);
 	g_value_unset (&value);
 
 	if (title == NULL)
 		return NULL;
 
 	bacon_video_widget_get_metadata (totem->bvw, BVW_INFO_ARTIST, &value);
-	artist = g_strdup (g_value_get_string (&value));
+	artist = g_value_dup_string (&value);
 	g_value_unset (&value);
 
 	if (artist == NULL)
@@ -573,7 +593,7 @@ update_mrl_label (Totem *totem, const char *name)
 		/* Update the mrl label */
 		text = g_strdup_printf
 			("<span size=\"medium\"><b>%s</b></span>",
-			 _("No file"));
+			 _("No File"));
 		widget = glade_xml_get_widget (totem->xml, "tcw_title_label");
 		gtk_label_set_markup (GTK_LABEL (widget), text);
 		widget = glade_xml_get_widget (totem->xml, "tmw_title_label");
@@ -601,9 +621,8 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 	}
 
 	/* Reset the properties and wait for the signal*/
-	bacon_video_widget_properties_update
-		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-		 totem->bvw, TRUE);
+	bacon_video_widget_properties_reset
+		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties));
 
 	if (mrl == NULL)
 	{
@@ -701,7 +720,6 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
 		gtk_widget_set_sensitive (widget, caps);
 
-
 		/* Take a screenshot */
 		widget = glade_xml_get_widget (totem->xml,
 				"tmw_take_screenshot_menu_item");
@@ -1122,37 +1140,6 @@ on_playlist_button_toggled (GtkToggleButton *button, Totem *totem)
 		gtk_widget_hide (GTK_WIDGET (totem->playlist));
 }
 
-static void
-on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
-                         Totem *totem)
-{
-	char *uri;
-	gboolean playlist_changed;
-	guint end;
-
-	uri = egg_recent_item_get_uri (item);
-
-	totem_signal_block_by_data (totem->playlist, totem);
-
-	end = totem_playlist_get_last (totem->playlist);
-	playlist_changed = totem_playlist_add_mrl (totem->playlist, uri, NULL);
-	egg_recent_model_add_full (totem->recent_model, item);
-
-	totem_signal_unblock_by_data (totem->playlist, totem);
-
-	if (playlist_changed)
-	{
-		char *mrl;
-
-		totem_playlist_set_current (totem->playlist, end + 1);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
-		g_free (mrl);
-	}
-
-	g_free (uri);
-}
-
 static void
 on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
@@ -1177,34 +1164,58 @@ on_channels_change_event (BaconVideoWidget *bvw, Totem *totem)
 	totem_sublang_update (totem);
 }
 
+static void
+on_playlist_change_name (TotemPlaylist *playlist, Totem *totem)
+{
+	char *name, *artist, *album, *title;
+	gboolean cur;
+
+	if ((name = totem_playlist_get_current_title (playlist,
+						      &cur)) != NULL) {
+		update_mrl_label (totem, name);
+		g_free (name);
+	}
+
+	if (totem_playlist_get_current_metadata (playlist, &artist,
+						 &title, &album) != FALSE) {
+		bacon_video_widget_properties_from_metadata (
+			BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
+			artist, title, album);
+
+		g_free (artist);
+		g_free (album);
+		g_free (title);
+	}
+}
+
 static void
 on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 {
         char *name = NULL;
-	gboolean custom;
 
         bacon_video_widget_properties_update
 		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-		 totem->bvw, FALSE);
+		 totem->bvw);
 
 	name = totem_get_nice_name_for_stream (totem);
 
-	if (name == NULL)
-	{
-		name = totem_playlist_get_current_title
-			(totem->playlist, &custom);
-		custom = TRUE;
-	} else {
-		custom = FALSE;
-	}
-
-	update_mrl_label (totem, name);
-
-	if (custom == FALSE)
+	if (name != NULL) {
 		totem_playlist_set_title
 			(TOTEM_PLAYLIST (totem->playlist), name);
+		g_free (name);
+	} else {
+		GValue v = { 0 };
+		const gchar *cdid;
 
-	g_free (name);
+		bacon_video_widget_get_metadata (totem->bvw, BVW_INFO_CDINDEX, &v);
+		if ((cdid = g_value_get_string (&v)) != NULL) {
+			totem_playlist_set_cdindex
+				(TOTEM_PLAYLIST (totem->playlist), cdid);
+		}
+		on_playlist_change_name
+			(TOTEM_PLAYLIST (totem->playlist), totem);
+		g_value_unset (&v);
+	}
 }
 
 static void
@@ -1283,6 +1294,11 @@ update_current_time (BaconVideoWidget *bvw,
 
 	if (totem->seek_lock == FALSE)
 	{
+		gtk_adjustment_set_value (totem->seekadj,
+				current_position * 65535);
+		gtk_adjustment_set_value (totem->fs_seekadj,
+				current_position * 65535);
+
 		if (stream_length == 0 && totem->mrl != NULL)
 		{
 			totem_statusbar_set_time_and_length
@@ -1295,11 +1311,6 @@ update_current_time (BaconVideoWidget *bvw,
 				(int) (stream_length / 1000));
 		}
 
-		gtk_adjustment_set_value (totem->seekadj,
-				current_position * 65535);
-		gtk_adjustment_set_value (totem->fs_seekadj,
-				current_position * 65535);
-
 		totem_time_label_set_time
 			(TOTEM_TIME_LABEL (totem->tcw_time_label),
 			 current_time, stream_length);
@@ -1429,31 +1440,6 @@ vol_cb (GtkWidget *widget, Totem *totem)
 	}
 }
 
-static void
-totem_action_add_recent (Totem *totem, const char *filename)
-{
-	EggRecentItem *item;
-
-	if (strstr (filename, "file:///") == NULL)
-		return;
-
-	item = egg_recent_item_new_from_uri (filename);
-	egg_recent_item_add_group (item, "Totem");
-	egg_recent_model_add_full (totem->recent_model, item);
-}
-
-static void
-totem_add_cd_track_name (Totem *totem, const char *filename)
-{
-	char *name;
-
-	bacon_video_widget_open (totem->bvw, filename, NULL);
-	name = totem_get_nice_name_for_stream (totem);
-	bacon_video_widget_close (totem->bvw);
-	totem_playlist_add_mrl (totem->playlist, filename, name);
-	g_free (name);
-}
-
 static gboolean
 totem_action_open_files (Totem *totem, char **list)
 {
@@ -1523,7 +1509,7 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 			} else if (strcmp (data, "cd:") == 0) {
 				totem_action_load_media (totem, MEDIA_TYPE_CDDA);
 			} else if (strstr (filename, "cdda:/") != NULL) {
-				totem_add_cd_track_name (totem, filename);
+				totem_playlist_add_mrl (totem->playlist, data, NULL);
 			} else if (totem_playlist_add_mrl (totem->playlist,
 						filename, NULL) != FALSE) {
 				totem_action_add_recent (totem, filename);
@@ -1551,7 +1537,7 @@ on_open1_activate (GtkButton *button, Totem *totem)
 	int response;
 	static char *path = NULL;
 
-	fs = gtk_file_chooser_dialog_new (_("Select files"),
+	fs = gtk_file_chooser_dialog_new (_("Select Files"),
 			GTK_WINDOW (totem->win), GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
@@ -1647,9 +1633,10 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 			mrl = totem_playlist_get_current_mrl (totem->playlist);
 			totem_action_set_mrl_and_play (totem, mrl);
 			g_free (mrl);
-
+#ifndef HAVE_GTK_ONLY
 			gnome_entry_append_history (GNOME_ENTRY (gentry),
 					TRUE, uri);
+#endif /* !HAVE_GTK_ONLY */
 		}
 	}
 
@@ -1657,37 +1644,6 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 	g_object_unref (glade);
 }
 
-static void
-on_play_disc1_activate (GtkButton *button, Totem *totem)
-{
-	MediaType type;
-	GError *error = NULL;
-	const gchar *device;
-
-	device = gconf_client_get_string (totem->gc,
-					  GCONF_PREFIX"/mediadev", NULL);
-	type = totem_cd_detect_type (device, &error);
-	switch (type) {
-		case MEDIA_TYPE_ERROR:
-			totem_action_error ("Failed to play Audio/Video Disc",
-					    error ? error->message : "Reason unknown",
-					    totem);
-			return;
-		case MEDIA_TYPE_DATA:
-			/* Maybe set default location to the mountpoint of
-			 * this device?... */
-			on_open1_activate (button, totem);
-			return;
-		case MEDIA_TYPE_DVD:
-		case MEDIA_TYPE_VCD:
-		case MEDIA_TYPE_CDDA:
-			totem_action_play_media (totem, type);
-			break;
-		default:
-			g_assert_not_reached ();
-	}
-}
-
 static void
 on_eject1_activate (GtkButton *button, Totem *totem)
 {
@@ -1820,9 +1776,6 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	GtkWidget *menubar, *controlbar, *statusbar, *item, *bvw_vbox, *widget;
 	int width = 0, height = 0;
 
-	if (totem->bvw == NULL)
-		return;
-
 	menubar = glade_xml_get_widget (totem->xml, "tmw_menubar");
 	controlbar = glade_xml_get_widget (totem->xml, "tmw_controls_vbox");
 	statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
@@ -2011,7 +1964,7 @@ on_take_screenshot1_activate (GtkButton *button, Totem *totem)
 	pixbuf = bacon_video_widget_get_current_frame (totem->bvw);
 	if (pixbuf == NULL)
 	{
-		totem_action_error (_("Totem could not get a screenshot of that film."), _("Please file a bug, this isn't supposed to happen."), totem);
+		totem_action_error (_("Totem could not get a screenshot of that film."), _("This is not supposed to happen; please file a bug report."), totem);
 		return;
 	}
 
@@ -2209,9 +2162,12 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_PLAY:
 		totem_action_play (totem);
 		break;
-	case TOTEM_REMOTE_COMMAND_PAUSE:
+	case TOTEM_REMOTE_COMMAND_PLAYPAUSE:
 		totem_action_play_pause (totem);
 		break;
+	case TOTEM_REMOTE_COMMAND_PAUSE:
+		totem_action_pause (totem);
+		break;
 	case TOTEM_REMOTE_COMMAND_SEEK_FORWARD:
 		totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET);
 		break;
@@ -2254,7 +2210,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		} else if (g_str_has_prefix (url, "cd:") != FALSE) {
 			totem_action_play_media (totem, MEDIA_TYPE_CDDA);
 		} else if (g_str_has_prefix (url, "cdda:/") != FALSE) {
-			totem_add_cd_track_name (totem, url);
+			totem_playlist_add_mrl (totem->playlist, url, NULL);
 		} else if (totem_playlist_add_mrl (totem->playlist,
 					url, NULL) != FALSE) {
 			totem_action_add_recent (totem, url);
@@ -2276,6 +2232,18 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 			gtk_check_menu_item_set_active (item, !value);
 		}
 		break;
+	case TOTEM_REMOTE_COMMAND_SHOW_PLAYING:
+		{
+			char *title;
+			gboolean custom;
+
+			title = totem_playlist_get_current_title
+				(totem->playlist, &custom);
+			bacon_message_connection_send (totem->conn,
+					title ? title : SHOW_PLAYING_NO_TRACKS);
+			g_free (title);
+		}
+		break;
 	default:
 		handled = FALSE;
 		break;
@@ -2596,6 +2564,7 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 		totem_gromit_clear (FALSE);
 		break;
 #endif /* !HAVE_GTK_ONLY */
+	case GDK_F11:
 	case GDK_f:
 	case GDK_F:
 		totem_action_fullscreen_toggle (totem);
@@ -2918,9 +2887,6 @@ totem_callback_connect (Totem *totem)
 			"tmw_open_location_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_open_location1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_play_disc_menu_item");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_play_disc1_activate), totem);
 	item = glade_xml_get_widget (totem->xml, "tmw_eject_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_eject1_activate), totem);
@@ -3400,39 +3366,6 @@ totem_time_display_create (void)
 	return widget;
 }
 
-static void
-totem_setup_recent (Totem *totem)
-{
-	GtkWidget *menu_item;
-	GtkWidget *menu;
-
-	menu_item = glade_xml_get_widget (totem->xml, "tmw_menu_item_movie");
-	menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
-	menu_item = glade_xml_get_widget (totem->xml,
-			"tmw_menu_recent_separator");
-
-	g_return_if_fail (menu != NULL);
-	g_return_if_fail (menu_item != NULL);
-
-	/* it would be better if we just filtered by mime-type, but there
-	 * doesn't seem to be an easy way to figure out which mime-types we
-	 * can handle */
-	totem->recent_model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
-
-	totem->recent_view = egg_recent_view_gtk_new (menu, menu_item);
-	egg_recent_view_gtk_show_icons (EGG_RECENT_VIEW_GTK
-			(totem->recent_view), FALSE);
-	egg_recent_model_set_limit (totem->recent_model, 5);
-	egg_recent_view_set_model (EGG_RECENT_VIEW (totem->recent_view),
-			totem->recent_model);
-	egg_recent_model_set_filter_groups (totem->recent_model,
-			"Totem", NULL);
-	egg_recent_view_gtk_set_trailing_sep (totem->recent_view, TRUE);
-
-	g_signal_connect (totem->recent_view, "activate",
-			G_CALLBACK (on_recent_file_activate), totem);
-}
-
 static void
 totem_message_connection_receive_cb (const char *msg, Totem *totem)
 {
@@ -3471,7 +3404,7 @@ main (int argc, char **argv)
 	{
 		gtk_init (&argc, &argv);
 		g_set_application_name (_("Totem Movie Player"));
-		totem_action_error_and_exit (_("Could not initialise the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
+		totem_action_error_and_exit (_("Could not initialize the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
 	}
 
 	g_thread_init (NULL);
@@ -3488,11 +3421,13 @@ main (int argc, char **argv)
 #endif /* HAVE_GTK_ONLY */
 
 	g_set_application_name (_("Totem Movie Player"));
+
 	gnome_vfs_init ();
 
-	if ((gc = gconf_client_get_default ()) == NULL)
+	gc = gconf_client_get_default ();
+	if (gc == NULL)
 	{
-		totem_action_error_and_exit (_("Totem couln't initialise the configuration engine."), _("Make sure that GNOME is properly installed."), NULL);
+		totem_action_error_and_exit (_("Totem could not initialize the configuration engine."), _("Make sure that GNOME is properly installed."), NULL);
 	}
 
 #ifndef HAVE_GTK_ONLY
@@ -3538,9 +3473,10 @@ main (int argc, char **argv)
 
 	/* The playlist */
 	totem->playlist = TOTEM_PLAYLIST (totem_playlist_new ());
-
 	if (totem->playlist == NULL)
 		totem_action_exit (totem);
+	g_signal_connect(totem->playlist, "active-name-changed",
+			 G_CALLBACK (on_playlist_change_name), totem);
 
 	/* The rest of the widgets */
 	totem->state = STATE_STOPPED;
@@ -3576,9 +3512,10 @@ main (int argc, char **argv)
 
 	totem_session_setup (totem, argv);
 	totem_setup_recent (totem);
+	totem_setup_file_monitoring (totem);
+	totem_setup_play_disc (totem);
 	totem_callback_connect (totem);
 	totem_setup_window (totem);
-	totem_setup_file_monitoring (totem);
 
 	/* Show ! gtk_main_iteration trickery to show all the widgets
 	 * we have so far */
